@@ -1,9 +1,9 @@
 /*
- * "$Id: lpr.c 7021 2007-10-10 22:50:07Z mike $"
+ * "$Id: lpr.c 7720 2008-07-11 22:46:21Z mike $"
  *
  *   "lpr" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -14,8 +14,7 @@
  *
  * Contents:
  *
- *   main()       - Parse options and send files for printing.
- *   sighandler() - Signal catcher for when we print from stdin...
+ *   main() - Parse options and send files for printing.
  */
 
 /*
@@ -30,25 +29,6 @@
 #include <cups/cups.h>
 #include <cups/i18n.h>
 
-#ifndef WIN32
-#  include <unistd.h>
-#  include <signal.h>
-
-
-/*
- * Local functions.
- */
-
-void	sighandler(int);
-#endif /* !WIN32 */
-
-
-/*
- * Globals...
- */
-
-char	tempfile[1024];		/* Temporary file for printing from stdin */
-
 
 /*
  * 'main()' - Parse options and send files for printing.
@@ -68,28 +48,18 @@ main(int  argc,				/* I - Number of command-line arguments */
   int		num_copies;		/* Number of copies per file */
   int		num_files;		/* Number of files to print */
   const char	*files[1000];		/* Files to print */
-  int		num_dests;		/* Number of destinations */
-  cups_dest_t	*dests,			/* Destinations */
-		*dest;			/* Selected destination */
+  cups_dest_t	*dest;			/* Selected destination */
   int		num_options;		/* Number of options */
   cups_option_t	*options;		/* Options */
   int		deletefile;		/* Delete file after print? */
   char		buffer[8192];		/* Copy buffer */
-  ssize_t	bytes;			/* Bytes copied */
-  off_t		filesize;		/* Size of temp file */
-  int		temp;			/* Temporary file descriptor */
-#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;		/* Signal action */
-  struct sigaction oldaction;		/* Old signal action */
-#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
   _cupsSetLocale(argv);
 
   deletefile  = 0;
   printer     = NULL;
-  num_dests   = 0;
-  dests       = NULL;
+  dest        = NULL;
   num_options = 0;
   options     = NULL;
   num_files   = 0;
@@ -258,10 +228,7 @@ main(int  argc,				/* I - Number of command-line arguments */
             if ((instance = strrchr(printer, '/')) != NULL)
 	      *instance++ = '\0';
 
-	    if (num_dests == 0)
-	      num_dests = cupsGetDests(&dests);
-
-            if ((dest = cupsGetDest(printer, instance, num_dests, dests)) != NULL)
+            if ((dest = cupsGetNamedDest(NULL, printer, instance)) != NULL)
 	    {
 	      for (j = 0; j < dest->num_options; j ++)
 	        if (cupsGetOption(dest->options[j].name, num_options,
@@ -355,10 +322,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (printer == NULL)
   {
-    if (num_dests == 0)
-      num_dests = cupsGetDests(&dests);
-
-    if ((dest = cupsGetDest(NULL, NULL, num_dests, dests)) != NULL)
+    if ((dest = cupsGetNamedDest(NULL, NULL, NULL)) != NULL)
     {
       printer = dest->name;
 
@@ -387,7 +351,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     else
       val = "LPDEST";
 
-    if (printer && !cupsGetDest(printer, NULL, num_dests, dests))
+    if (printer && !cupsGetNamedDest(NULL, printer, NULL))
       _cupsLangPrintf(stderr,
                       _("%s: Error - %s environment variable names "
 		        "non-existent destination \"%s\"!\n"),
@@ -418,72 +382,38 @@ main(int  argc,				/* I - Number of command-line arguments */
         unlink(files[i]);
     }
   }
-  else
+  else if ((job_id = cupsCreateJob(CUPS_HTTP_DEFAULT, printer,
+                                   title ? title : "(stdin)",
+                                   num_options, options)) > 0)
   {
-    num_files = 1;
+    http_status_t	status;		/* Write status */
+    const char		*format;	/* Document format */
+    ssize_t		bytes;		/* Bytes read */
 
-#ifndef WIN32
-#  if defined(HAVE_SIGSET)
-    sigset(SIGHUP, sighandler);
-    if (sigset(SIGINT, sighandler) == SIG_IGN)
-      sigset(SIGINT, SIG_IGN);
-    sigset(SIGTERM, sighandler);
-#  elif defined(HAVE_SIGACTION)
-    memset(&action, 0, sizeof(action));
-    action.sa_handler = sighandler;
 
-    sigaction(SIGHUP, &action, NULL);
-    sigaction(SIGINT, NULL, &oldaction);
-    if (oldaction.sa_handler != SIG_IGN)
-      sigaction(SIGINT, &action, NULL);
-    sigaction(SIGTERM, &action, NULL);
-#  else
-    signal(SIGHUP, sighandler);
-    if (signal(SIGINT, sighandler) == SIG_IGN)
-      signal(SIGINT, SIG_IGN);
-    signal(SIGTERM, sighandler);
-#  endif
-#endif /* !WIN32 */
+    if (cupsGetOption("raw", num_options, options))
+      format = CUPS_FORMAT_RAW;
+    else if ((format = cupsGetOption("document-format", num_options,
+                                     options)) == NULL)
+      format = CUPS_FORMAT_AUTO;
 
-    if ((temp = cupsTempFd(tempfile, sizeof(tempfile))) < 0)
+    status = cupsStartDocument(CUPS_HTTP_DEFAULT, printer, job_id, NULL,
+                               format, 1);
+
+    while (status == HTTP_CONTINUE &&
+           (bytes = read(0, buffer, sizeof(buffer))) > 0)
+      status = cupsWriteRequestData(CUPS_HTTP_DEFAULT, buffer, bytes);
+
+    if (status != HTTP_CONTINUE)
     {
       _cupsLangPrintf(stderr,
-                      _("%s: Error - unable to create temporary file "
-		        "\"%s\" - %s\n"),
-        	      argv[0], tempfile, strerror(errno));
+		      _("%s: Error - unable to queue from stdin - %s\n"),
+		      argv[0], httpStatus(status));
       return (1);
     }
 
-    while ((bytes = read(0, buffer, sizeof(buffer))) > 0)
-      if (write(temp, buffer, bytes) < 0)
-      {
-	_cupsLangPrintf(stderr,
-	                _("%s: Error - unable to write to temporary file "
-			  "\"%s\" - %s\n"),
-        		argv[0], tempfile, strerror(errno));
-        close(temp);
-        unlink(tempfile);
-	return (1);
-      }
-
-    filesize = lseek(temp, 0, SEEK_CUR);
-    close(temp);
-
-    if (filesize <= 0)
-    {
-      _cupsLangPrintf(stderr,
-                      _("%s: Error - stdin is empty, so no job has been sent.\n"),
-		      argv[0]);
-      unlink(tempfile);
-      return (1);
-    }
-
-    if (title)
-      job_id = cupsPrintFile(printer, tempfile, title, num_options, options);
-    else
-      job_id = cupsPrintFile(printer, tempfile, "(stdin)", num_options, options);
-
-    unlink(tempfile);
+    if (cupsFinishDocument(CUPS_HTTP_DEFAULT, printer) != IPP_OK)
+      job_id = 0;
   }
 
   if (job_id < 1)
@@ -496,29 +426,6 @@ main(int  argc,				/* I - Number of command-line arguments */
 }
 
 
-#ifndef WIN32
 /*
- * 'sighandler()' - Signal catcher for when we print from stdin...
- */
-
-void
-sighandler(int s)			/* I - Signal number */
-{
- /*
-  * Remove the temporary file we're using to print from stdin...
-  */
-
-  unlink(tempfile);
-
- /*
-  * Exit...
-  */
-
-  exit(s);
-}
-#endif /* !WIN32 */
-
-
-/*
- * End of "$Id: lpr.c 7021 2007-10-10 22:50:07Z mike $".
+ * End of "$Id: lpr.c 7720 2008-07-11 22:46:21Z mike $".
  */
