@@ -1,5 +1,5 @@
 /*
- * "$Id: ipp-var.c 8859 2009-11-09 23:01:17Z mike $"
+ * "$Id: ipp-var.c 9181 2010-06-22 21:42:44Z mike $"
  *
  *   CGI <-> IPP variable routines for the Common UNIX Printing System (CUPS).
  *
@@ -285,10 +285,14 @@ cgiMoveJobs(http_t     *http,		/* I - Connection to server */
 
 
  /*
-  * See who is logged in...
+  * Make sure we have a username...
   */
 
-  user = getenv("REMOTE_USER");
+  if ((user = getenv("REMOTE_USER")) == NULL)
+  {
+    puts("Status: 401\n");
+    exit(0);
+  }
 
  /*
   * See if the user has already selected a new destination...
@@ -333,7 +337,7 @@ cgiMoveJobs(http_t     *http,		/* I - Connection to server */
                    NULL, job_uri);
       ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
                    "requested-attributes", NULL, "job-printer-uri");
-      
+
       if ((response = cupsDoRequest(http, request, "/")) != NULL)
       {
         if ((attr = ippFindAttribute(response, "job-printer-uri",
@@ -926,7 +930,7 @@ cgiRewriteURL(const char *uri,		/* I - Current URI */
 		 ishttps ? "https" : "http",
 		 userpass, hostname, port, resource);
       else
-	snprintf(url, urlsize, "%s://%s:%d%s", 
+	snprintf(url, urlsize, "%s://%s:%d%s",
 		 ishttps ? "https" : "http",
 		 hostname, port, resource);
     }
@@ -1518,7 +1522,11 @@ cgiShowJobs(http_t     *http,		/* I - Connection to server */
     */
 
     if (dest)
+    {
       snprintf(val, sizeof(val), "/%s/%s", section, dest);
+      cgiSetVariable("PRINTER_NAME", dest);
+      cgiSetVariable("PRINTER_URI_SUPPORTED", val);
+    }
     else
       strlcpy(val, "/jobs/", sizeof(val));
 
@@ -1580,5 +1588,5 @@ cgiText(const char *message)		/* I - Message */
 
 
 /*
- * End of "$Id: ipp-var.c 8859 2009-11-09 23:01:17Z mike $".
+ * End of "$Id: ipp-var.c 9181 2010-06-22 21:42:44Z mike $".
  */
