@@ -1,10 +1,10 @@
 /*
- * "$Id: adminutil.c 7264 2008-01-28 23:19:27Z mike $"
+ * "$Id: adminutil.c 9376 2010-11-17 19:58:22Z mike $"
  *
  *   Administration utility API definitions for the Common UNIX Printing
  *   System (CUPS).
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 2001-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -17,16 +17,14 @@
  *
  * Contents:
  *
- *   cupsAdminCreateWindowsPPD()   - Create the Windows PPD file for a printer.
- *   cupsAdminExportSamba()        - Export a printer to Samba.
- *   cupsAdminGetServerSettings()  - Get settings from the server.
- *   _cupsAdminGetServerSettings() - Get settings from the server (private).
- *   cupsAdminSetServerSettings()  - Set settings on the server.
- *   _cupsAdminSetServerSettings() - Set settings on the server (private).
- *   do_samba_command()            - Do a SAMBA command.
- *   get_cupsd_conf()              - Get the current cupsd.conf file.
- *   invalidate_cupsd_cache()      - Invalidate the cached cupsd.conf settings.
- *   write_option()                - Write a CUPS option to a PPD file.
+ *   cupsAdminCreateWindowsPPD()  - Create the Windows PPD file for a printer.
+ *   cupsAdminExportSamba()       - Export a printer to Samba.
+ *   cupsAdminGetServerSettings() - Get settings from the server.
+ *   cupsAdminSetServerSettings() - Set settings on the server.
+ *   do_samba_command()           - Do a SAMBA command.
+ *   get_cupsd_conf()             - Get the current cupsd.conf file.
+ *   invalidate_cupsd_cache()     - Invalidate the cached cupsd.conf settings.
+ *   write_option()               - Write a CUPS option to a PPD file.
  */
 
 /*
@@ -50,12 +48,6 @@
  * Local functions...
  */
 
-extern int		_cupsAdminGetServerSettings(http_t *http,
-				                    int *num_settings,
-			                            cups_option_t **settings);
-extern int		_cupsAdminSetServerSettings(http_t *http,
-			                            int num_settings,
-			                            cups_option_t *settings);
 static int		do_samba_command(const char *command,
 			                 const char *address,
 			                 const char *subcommand,
@@ -75,11 +67,13 @@ static void		write_option(cups_file_t *dstfp, int order,
 
 /*
  * 'cupsAdminCreateWindowsPPD()' - Create the Windows PPD file for a printer.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 char *					/* O - PPD file or NULL */
 cupsAdminCreateWindowsPPD(
-    http_t     *http,			/* I - Connection to server */
+    http_t     *http,			/* I - Connection to server or @code CUPS_HTTP_DEFAULT@ */
     const char *dest,			/* I - Printer or class */
     char       *buffer,			/* I - Filename buffer */
     int        bufsize)			/* I - Size of filename buffer */
@@ -121,6 +115,9 @@ cupsAdminCreateWindowsPPD(
   if (buffer)
     *buffer = '\0';
 
+  if (!http)
+    http = _cupsConnect();
+
   if (!http || !dest || !buffer || bufsize < 2)
     return (NULL);
 
@@ -238,7 +235,7 @@ cupsAdminCreateWindowsPPD(
     }
     else if (jcloption && !strncmp(line, "*OrderDependency:", 17))
     {
-      for (ptr = line + 17; *ptr && isspace(*ptr & 255); ptr ++);
+      for (ptr = line + 17; _cups_isspace(*ptr); ptr ++);
 
       ptr = strchr(ptr, ' ');
 
@@ -257,9 +254,9 @@ cupsAdminCreateWindowsPPD(
       if ((ptr = strchr(line, ':')) == NULL)
       {
         snprintf(line, sizeof(line),
-	          _cupsLangString(language, _("Missing value on line %d!")),
-		  linenum);
-        _cupsSetError(IPP_DOCUMENT_FORMAT_ERROR, line);
+	         _cupsLangString(language, _("Missing value on line %d!")),
+		 linenum);
+        _cupsSetError(IPP_DOCUMENT_FORMAT_ERROR, line, 0);
 
         cupsFileClose(srcfp);
         cupsFileClose(dstfp);
@@ -278,7 +275,7 @@ cupsAdminCreateWindowsPPD(
 	         _cupsLangString(language,
 		                 _("Missing double quote on line %d!")),
 	         linenum);
-        _cupsSetError(IPP_DOCUMENT_FORMAT_ERROR, line);
+        _cupsSetError(IPP_DOCUMENT_FORMAT_ERROR, line, 0);
 
         cupsFileClose(srcfp);
         cupsFileClose(dstfp);
@@ -297,7 +294,7 @@ cupsAdminCreateWindowsPPD(
 	         _cupsLangString(language,
 		                 _("Bad option + choice on line %d!")),
 	         linenum);
-        _cupsSetError(IPP_DOCUMENT_FORMAT_ERROR, line);
+        _cupsSetError(IPP_DOCUMENT_FORMAT_ERROR, line, 0);
 
         cupsFileClose(srcfp);
         cupsFileClose(dstfp);
@@ -340,10 +337,7 @@ cupsAdminCreateWindowsPPD(
 
   if (linenum == 0)
   {
-    snprintf(line, sizeof(line),
-             _cupsLangString(language, _("Empty PPD file!")),
-             linenum);
-    _cupsSetError(IPP_DOCUMENT_FORMAT_ERROR, line);
+    _cupsSetError(IPP_DOCUMENT_FORMAT_ERROR, _("Empty PPD file!"), 1);
 
     cupsFileClose(dstfp);
     unlink(buffer);
@@ -386,7 +380,7 @@ cupsAdminCreateWindowsPPD(
   {
     write_option(dstfp, jclorder ++, "cupsJobSheetsStart", "Start Banner",
                  "job-sheets", suppattr, defattr, 0, 2);
-    write_option(dstfp, jclorder ++, "cupsJobSheetsEnd", "End Banner",
+    write_option(dstfp, jclorder, "cupsJobSheetsEnd", "End Banner",
                  "job-sheets", suppattr, defattr, 1, 2);
   }
 
@@ -401,6 +395,8 @@ cupsAdminCreateWindowsPPD(
 
 /*
  * 'cupsAdminExportSamba()' - Export a printer to Samba.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 int					/* O - 1 on success, 0 on failure */
@@ -431,7 +427,7 @@ cupsAdminExportSamba(
 
   if (!dest || !ppd || !samba_server || !samba_user || !samba_password)
   {
-    _cupsSetError(IPP_INTERNAL_ERROR, NULL);
+    _cupsSetError(IPP_INTERNAL_ERROR, strerror(EINVAL), 0);
     return (0);
   }
 
@@ -441,7 +437,7 @@ cupsAdminExportSamba(
 
   if ((fp = cupsTempFile2(authfile, sizeof(authfile))) == NULL)
   {
-    _cupsSetError(IPP_INTERNAL_ERROR, strerror(errno));
+    _cupsSetError(IPP_INTERNAL_ERROR, NULL, 0);
     return (0);
   }
 
@@ -494,7 +490,7 @@ cupsAdminExportSamba(
 	                       _("Unable to copy Windows 2000 printer "
 	                         "driver files (%d)!")), status);
 
-      _cupsSetError(IPP_INTERNAL_ERROR, message);
+      _cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
       if (logfile)
 	_cupsLangPrintf(logfile, "%s\n", message);
@@ -529,7 +525,7 @@ cupsAdminExportSamba(
 	                         _("Unable to copy CUPS printer driver "
 				   "files (%d)!")), status);
 
-	_cupsSetError(IPP_INTERNAL_ERROR, message);
+	_cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
 	if (logfile)
 	  _cupsLangPrintf(logfile, "%s\n", message);
@@ -572,7 +568,7 @@ cupsAdminExportSamba(
                 	       _("Unable to install Windows 2000 printer "
 		        	 "driver files (%d)!")), status);
 
-      _cupsSetError(IPP_INTERNAL_ERROR, message);
+      _cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
       if (logfile)
 	_cupsLangPrintf(logfile, "%s\n", message);
@@ -617,7 +613,7 @@ cupsAdminExportSamba(
                 	       _("Unable to copy Windows 9x printer "
 		        	 "driver files (%d)!")), status);
 
-      _cupsSetError(IPP_INTERNAL_ERROR, message);
+      _cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
       if (logfile)
 	_cupsLangPrintf(logfile, "%s\n", message);
@@ -646,7 +642,7 @@ cupsAdminExportSamba(
                 	       _("Unable to install Windows 9x printer "
 		        	 "driver files (%d)!")), status);
 
-      _cupsSetError(IPP_INTERNAL_ERROR, message);
+      _cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
       if (logfile)
 	_cupsLangPrintf(logfile, "%s\n", message);
@@ -698,7 +694,7 @@ cupsAdminExportSamba(
 	                       _("Unable to copy 64-bit Windows printer "
 	                         "driver files (%d)!")), status);
 
-      _cupsSetError(IPP_INTERNAL_ERROR, message);
+      _cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
       if (logfile)
 	_cupsLangPrintf(logfile, "%s\n", message);
@@ -733,7 +729,7 @@ cupsAdminExportSamba(
 	                         _("Unable to copy 64-bit CUPS printer driver "
 				   "files (%d)!")), status);
 
-	_cupsSetError(IPP_INTERNAL_ERROR, message);
+	_cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
 	if (logfile)
 	  _cupsLangPrintf(logfile, "%s\n", message);
@@ -776,7 +772,7 @@ cupsAdminExportSamba(
                 	       _("Unable to install Windows 2000 printer "
 		        	 "driver files (%d)!")), status);
 
-      _cupsSetError(IPP_INTERNAL_ERROR, message);
+      _cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
       if (logfile)
 	_cupsLangPrintf(logfile, "%s\n", message);
@@ -801,13 +797,16 @@ cupsAdminExportSamba(
 				"are installed!")),
               sizeof(message));
 
-    _cupsSetError(IPP_NOT_FOUND, message);
+    _cupsSetError(IPP_NOT_FOUND, message, 0);
     _cupsLangPrintf(logfile, "%s\n", message);
   }
 
   if (have_drivers == 0)
   {
-    _cupsSetError(IPP_NOT_FOUND, message);
+    _cupsSetError(IPP_NOT_FOUND, message, 0);
+
+    unlink(authfile);
+
     return (0);
   }
 
@@ -825,7 +824,7 @@ cupsAdminExportSamba(
         		     _("Unable to set Windows printer driver (%d)!")),
         		     status);
 
-    _cupsSetError(IPP_INTERNAL_ERROR, message);
+    _cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
     if (logfile)
       _cupsLangPrintf(logfile, "%s\n", message);
@@ -847,31 +846,12 @@ cupsAdminExportSamba(
  * The returned settings should be freed with cupsFreeOptions() when
  * you are done with them.
  *
- * @since CUPS 1.3@
+ * @since CUPS 1.3/Mac OS X 10.5@
  */
 
 int					/* O - 1 on success, 0 on failure */
 cupsAdminGetServerSettings(
-    http_t        *http,		/* I - Connection to server */
-    int           *num_settings,	/* O - Number of settings */
-    cups_option_t **settings)		/* O - Settings */
-{
-  return (_cupsAdminGetServerSettings(http, num_settings, settings));
-}
-
-
-/*
- * '_cupsAdminGetServerSettings()' - Get settings from the server.
- *
- * The returned settings should be freed with cupsFreeOptions() when
- * you are done with them.
- *
- * @since CUPS 1.2@
- */
-
-int					/* O - 1 on success, 0 on failure */
-_cupsAdminGetServerSettings(
-    http_t        *http,		/* I - Connection to server */
+    http_t        *http,		/* I - Connection to server or @code CUPS_HTTP_DEFAULT@ */
     int           *num_settings,	/* O - Number of settings */
     cups_option_t **settings)		/* O - Settings */
 {
@@ -890,9 +870,65 @@ _cupsAdminGetServerSettings(
   * Range check input...
   */
 
+  if (!http)
+  {
+   /*
+    * See if we are connected to the same server...
+    */
+
+    if (cg->http)
+    {
+     /*
+      * Compare the connection hostname, port, and encryption settings to
+      * the cached defaults; these were initialized the first time we
+      * connected...
+      */
+
+      if (strcmp(cg->http->hostname, cg->server) ||
+          cg->ipp_port != _httpAddrPort(cg->http->hostaddr) ||
+	  (cg->http->encryption != cg->encryption &&
+	   cg->http->encryption == HTTP_ENCRYPT_NEVER))
+      {
+       /*
+	* Need to close the current connection because something has changed...
+	*/
+
+	httpClose(cg->http);
+	cg->http = NULL;
+      }
+    }
+
+   /*
+    * (Re)connect as needed...
+    */
+
+    if (!cg->http)
+    {
+      if ((cg->http = _httpCreate(cupsServer(), ippPort(),
+                                  cupsEncryption())) == NULL)
+      {
+	if (errno)
+	  _cupsSetError(IPP_SERVICE_UNAVAILABLE, NULL, 0);
+	else
+	  _cupsSetError(IPP_SERVICE_UNAVAILABLE,
+			_("Unable to connect to host."), 1);
+
+	if (num_settings)
+	  *num_settings = 0;
+
+	if (settings)
+	  *settings = NULL;
+
+	return (0);
+      }
+    }
+
+    http = cg->http;
+  }
+
   if (!http || !num_settings || !settings)
   {
-    _cupsSetError(IPP_INTERNAL_ERROR, NULL);
+    _cupsSetError(IPP_INTERNAL_ERROR, strerror(EINVAL), 0);
 
     if (num_settings)
       *num_settings = 0;
@@ -919,9 +955,9 @@ _cupsAdminGetServerSettings(
 
 
       snprintf(message, sizeof(message),
-               _cupsLangString(cupsLangDefault(), _("open of %s failed: %s")),
+               _cupsLangString(cupsLangDefault(), _("Open of %s failed: %s")),
                cupsdconf, strerror(errno));
-      _cupsSetError(IPP_INTERNAL_ERROR, message);
+      _cupsSetError(IPP_INTERNAL_ERROR, message, 0);
     }
   }
   else
@@ -958,7 +994,7 @@ _cupsAdminGetServerSettings(
       if (!value && strncmp(line, "</", 2))
         value = line + strlen(line);
 
-      if (!strcasecmp(line, "Port") || !strcasecmp(line, "Listen"))
+      if ((!strcasecmp(line, "Port") || !strcasecmp(line, "Listen")) && value)
       {
 	char	*port;			/* Pointer to port number, if any */
 
@@ -980,7 +1016,7 @@ _cupsAdminGetServerSettings(
             && *value != '/'
 #endif /* AF_LOCAL */
 #ifdef AF_INET6
-            && strcmp(value, "::1")
+            && strcmp(value, "[::1]")
 #endif /* AF_INET6 */
 	    )
 	  remote_access = 1;
@@ -1014,7 +1050,7 @@ _cupsAdminGetServerSettings(
       {
 	in_policy = 0;
       }
-      else if (!strcasecmp(line, "<Limit") && in_policy)
+      else if (!strcasecmp(line, "<Limit") && in_policy && value)
       {
        /*
 	* See if the policy limit is for the Cancel-Job operation...
@@ -1025,7 +1061,7 @@ _cupsAdminGetServerSettings(
 
 	while (*value)
 	{
-	  for (valptr = value; !isspace(*valptr & 255) && *valptr; valptr ++);
+	  for (valptr = value; *valptr && !_cups_isspace(*valptr); valptr ++);
 
 	  if (*valptr)
 	    *valptr++ = '\0';
@@ -1036,7 +1072,7 @@ _cupsAdminGetServerSettings(
 	    break;
 	  }
 
-          for (value = valptr; isspace(*value & 255); value ++);
+          for (value = valptr; _cups_isspace(*value); value ++);
 	}
       }
       else if (!strcasecmp(line, "</Limit>"))
@@ -1047,7 +1083,7 @@ _cupsAdminGetServerSettings(
       {
 	cancel_policy = 0;
       }
-      else if (!strcasecmp(line, "<Location"))
+      else if (!strcasecmp(line, "<Location") && value)
       {
         in_admin_location = !strcasecmp(value, "/admin");
 	in_location       = 1;
@@ -1057,7 +1093,7 @@ _cupsAdminGetServerSettings(
 	in_admin_location = 0;
 	in_location       = 0;
       }
-      else if (!strcasecmp(line, "Allow") &&
+      else if (!strcasecmp(line, "Allow") && value &&
                strcasecmp(value, "localhost") && strcasecmp(value, "127.0.0.1")
 #ifdef AF_LOCAL
 	       && *value != '/'
@@ -1072,7 +1108,13 @@ _cupsAdminGetServerSettings(
         else if (!strcasecmp(value, "all"))
 	  remote_any = 1;
       }
-      else if (line[0] != '<' && !in_location && !in_policy)
+      else if (line[0] != '<' && !in_location && !in_policy &&
+	       strcasecmp(line, "Allow") &&
+               strcasecmp(line, "AuthType") &&
+	       strcasecmp(line, "Deny") &&
+	       strcasecmp(line, "Order") &&
+	       strcasecmp(line, "Require") &&
+	       strcasecmp(line, "Satisfy"))
         cg->cupsd_num_settings = cupsAddOption(line, value,
 	                                       cg->cupsd_num_settings,
 					       &(cg->cupsd_settings));
@@ -1136,28 +1178,12 @@ _cupsAdminGetServerSettings(
 /*
  * 'cupsAdminSetServerSettings()' - Set settings on the server.
  *
- * @since CUPS 1.3@
+ * @since CUPS 1.3/Mac OS X 10.5@
  */
 
 int					/* O - 1 on success, 0 on failure */
 cupsAdminSetServerSettings(
-    http_t        *http,		/* I - Connection to server */
-    int           num_settings,		/* I - Number of settings */
-    cups_option_t *settings)		/* I - Settings */
-{
-  return (_cupsAdminSetServerSettings(http, num_settings, settings));
-}
-
-
-/*
- * '_cupsAdminSetServerSettings()' - Set settings on the server.
- *
- * @since CUPS 1.2@
- */
-
-int					/* O - 1 on success, 0 on failure */
-_cupsAdminSetServerSettings(
-    http_t        *http,		/* I - Connection to server */
+    http_t        *http,		/* I - Connection to server or @code CUPS_HTTP_DEFAULT@ */
     int           num_settings,		/* I - Number of settings */
     cups_option_t *settings)		/* I - Settings */
 {
@@ -1210,9 +1236,12 @@ _cupsAdminSetServerSettings(
   * Range check input...
   */
 
+  if (!http)
+    http = _cupsConnect();
+
   if (!http || !num_settings || !settings)
   {
-    _cupsSetError(IPP_INTERNAL_ERROR, NULL);
+    _cupsSetError(IPP_INTERNAL_ERROR, strerror(EINVAL), 0);
 
     return (0);
   }
@@ -1221,12 +1250,12 @@ _cupsAdminSetServerSettings(
   * Get the cupsd.conf file...
   */
 
-  if ((status = get_cupsd_conf(http, cg, 0, cupsdconf, sizeof(cupsdconf),
-                               &remote)) == HTTP_OK)
+  if (get_cupsd_conf(http, cg, 0, cupsdconf, sizeof(cupsdconf),
+                     &remote) == HTTP_OK)
   {
     if ((cupsd = cupsFileOpen(cupsdconf, "r")) == NULL)
     {
-      _cupsSetError(IPP_INTERNAL_ERROR, strerror(errno));
+      _cupsSetError(IPP_INTERNAL_ERROR, NULL, 0);
       return (0);
     }
   }
@@ -1237,8 +1266,8 @@ _cupsAdminSetServerSettings(
   * Get current settings...
   */
 
-  if (!_cupsAdminGetServerSettings(http, &cupsd_num_settings,
-                                   &cupsd_settings))
+  if (!cupsAdminGetServerSettings(http, &cupsd_num_settings,
+				  &cupsd_settings))
     return (0);
 
   if ((val = cupsGetOption(CUPS_SERVER_DEBUG_LOGGING, cupsd_num_settings,
@@ -1247,36 +1276,54 @@ _cupsAdminSetServerSettings(
   else
     old_debug_logging = 0;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: old debug_logging=%d",
+                old_debug_logging));
+
   if ((val = cupsGetOption(CUPS_SERVER_REMOTE_ADMIN, cupsd_num_settings,
                            cupsd_settings)) != NULL)
     old_remote_admin = atoi(val);
   else
     old_remote_admin = 0;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: old remote_admin=%d",
+                old_remote_admin));
+
   if ((val = cupsGetOption(CUPS_SERVER_REMOTE_ANY, cupsd_num_settings,
                            cupsd_settings)) != NULL)
     remote_any = atoi(val);
   else
     remote_any = 0;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: old remote_any=%d",
+                remote_any));
+
   if ((val = cupsGetOption(CUPS_SERVER_REMOTE_PRINTERS, cupsd_num_settings,
                            cupsd_settings)) != NULL)
     old_remote_printers = atoi(val);
   else
     old_remote_printers = 1;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: old remote_printers=%d",
+                old_remote_printers));
+
   if ((val = cupsGetOption(CUPS_SERVER_SHARE_PRINTERS, cupsd_num_settings,
                            cupsd_settings)) != NULL)
     old_share_printers = atoi(val);
   else
     old_share_printers = 0;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: old share_printers=%d",
+                old_share_printers));
+
   if ((val = cupsGetOption(CUPS_SERVER_USER_CANCEL_ANY, cupsd_num_settings,
                            cupsd_settings)) != NULL)
     old_user_cancel_any = atoi(val);
   else
     old_user_cancel_any = 0;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: old user_cancel_any=%d",
+                old_user_cancel_any));
+
   cupsFreeOptions(cupsd_num_settings, cupsd_settings);
 
  /*
@@ -1300,16 +1347,22 @@ _cupsAdminSetServerSettings(
   else
     debug_logging = -1;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: debug_logging=%d",
+                debug_logging));
+
   if ((val = cupsGetOption(CUPS_SERVER_REMOTE_ANY, num_settings,
                            settings)) != NULL)
     remote_any = atoi(val);
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: remote_any=%d",
+                remote_any));
+
   if ((val = cupsGetOption(CUPS_SERVER_REMOTE_ADMIN, num_settings,
                            settings)) != NULL)
   {
     remote_admin = atoi(val);
 
-    if (remote_admin == old_remote_admin && remote_any < 0)
+    if (remote_admin == old_remote_admin)
     {
      /*
       * No change to this setting...
@@ -1321,6 +1374,9 @@ _cupsAdminSetServerSettings(
   else
     remote_admin = -1;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: remote_admin=%d",
+                remote_admin));
+
   if ((val = cupsGetOption(CUPS_SERVER_REMOTE_PRINTERS, num_settings,
                            settings)) != NULL)
   {
@@ -1338,12 +1394,15 @@ _cupsAdminSetServerSettings(
   else
     remote_printers = -1;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: remote_printers=%d",
+                remote_printers));
+
   if ((val = cupsGetOption(CUPS_SERVER_SHARE_PRINTERS, num_settings,
                            settings)) != NULL)
   {
     share_printers = atoi(val);
 
-    if (share_printers == old_share_printers && remote_any < 0)
+    if (share_printers == old_share_printers)
     {
      /*
       * No change to this setting...
@@ -1355,6 +1414,9 @@ _cupsAdminSetServerSettings(
   else
     share_printers = -1;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: share_printers=%d",
+                share_printers));
+
   if ((val = cupsGetOption(CUPS_SERVER_USER_CANCEL_ANY, num_settings,
                            settings)) != NULL)
   {
@@ -1372,6 +1434,9 @@ _cupsAdminSetServerSettings(
   else
     user_cancel_any = -1;
 
+  DEBUG_printf(("1cupsAdminSetServerSettings: user_cancel_any=%d",
+                user_cancel_any));
+
  /*
   * Create a temporary file for the new cupsd.conf file...
   */
@@ -1383,7 +1448,7 @@ _cupsAdminSetServerSettings(
     if (remote)
       unlink(cupsdconf);
 
-    _cupsSetError(IPP_INTERNAL_ERROR, strerror(errno));
+    _cupsSetError(IPP_INTERNAL_ERROR, NULL, 0);
     return (0);
   }
 
@@ -1423,13 +1488,13 @@ _cupsAdminSetServerSettings(
   while (cupsFileGetConf(cupsd, line, sizeof(line), &value, &linenum))
   {
     if ((!strcasecmp(line, "Port") || !strcasecmp(line, "Listen")) &&
-        (share_printers >= 0 || remote_admin >= 0))
+        (remote_admin >= 0 || remote_any > 0 || share_printers >= 0))
     {
       if (!wrote_port_listen)
       {
         wrote_port_listen = 1;
 
-	if (share_printers > 0 || remote_admin > 0)
+	if (remote_admin > 0 || remote_any > 0 || share_printers > 0)
 	{
 	  cupsFilePuts(temp, "# Allow remote access\n");
 	  cupsFilePrintf(temp, "Port %d\n", server_port);
@@ -1458,6 +1523,8 @@ _cupsAdminSetServerSettings(
               !strcasecmp(line, "BrowseAddress") ||
               !strcasecmp(line, "BrowseAllow") ||
               !strcasecmp(line, "BrowseDeny") ||
+              !strcasecmp(line, "BrowseLocalProtocols") ||
+              !strcasecmp(line, "BrowseRemoteProtocols") ||
               !strcasecmp(line, "BrowseOrder")) &&
 	     (remote_printers >= 0 || share_printers >= 0))
     {
@@ -1474,6 +1541,19 @@ _cupsAdminSetServerSettings(
 
         if (new_remote_printers || new_share_printers)
 	{
+	  const char *localp = cupsGetOption("BrowseLocalProtocols",
+					     num_settings, settings);
+	  const char *remotep = cupsGetOption("BrowseRemoteProtocols",
+					      num_settings, settings);
+
+          if (!localp || !localp[0])
+	    localp = cupsGetOption("BrowseLocalProtocols", cupsd_num_settings,
+	                           cupsd_settings);
+
+          if (!remotep)
+	    remotep = cupsGetOption("BrowseRemoteProtocols", cupsd_num_settings,
+	                            cupsd_settings);
+
 	  if (new_remote_printers && new_share_printers)
 	    cupsFilePuts(temp,
 	                 "# Enable printer sharing and shared printers.\n");
@@ -1488,10 +1568,36 @@ _cupsAdminSetServerSettings(
 	  cupsFilePuts(temp, "BrowseOrder allow,deny\n");
 
 	  if (new_remote_printers)
+	  {
 	    cupsFilePuts(temp, "BrowseAllow all\n");
 
+	    if (!remotep)
+	      remotep = CUPS_DEFAULT_BROWSE_REMOTE_PROTOCOLS;
+
+	    cupsFilePrintf(temp, "BrowseRemoteProtocols %s\n", remotep);
+          }
+	  else
+	    cupsFilePuts(temp, "BrowseRemoteProtocols\n");
+
+	  cupsd_num_settings = cupsAddOption("BrowseRemoteProtocols", remotep,
+					     cupsd_num_settings,
+					     &cupsd_settings);
+
 	  if (new_share_printers)
+	  {
 	    cupsFilePuts(temp, "BrowseAddress @LOCAL\n");
+
+	    if (!localp)
+	      localp = CUPS_DEFAULT_BROWSE_LOCAL_PROTOCOLS;
+
+	    cupsFilePrintf(temp, "BrowseLocalProtocols %s\n", localp);
+	  }
+	  else
+	    cupsFilePuts(temp, "BrowseLocalProtocols\n");
+
+	  cupsd_num_settings = cupsAddOption("BrowseLocalProtocols", localp,
+					     cupsd_num_settings,
+					     &cupsd_settings);
         }
 	else
 	{
@@ -1514,7 +1620,7 @@ _cupsAdminSetServerSettings(
       else
       {
         cupsFilePuts(temp, "# Show general information in error_log.\n");
-	cupsFilePuts(temp, "LogLevel info\n");
+	cupsFilePuts(temp, "LogLevel " CUPS_DEFAULT_LOG_LEVEL "\n");
       }
     }
     else if (!strcasecmp(line, "<Policy"))
@@ -1538,7 +1644,7 @@ _cupsAdminSetServerSettings(
 	                     "  <Limit Cancel-Job>\n"
 	                     "    Order deny,allow\n"
 			     "    Require user @OWNER "
-			     CUPS_DEFAULT_PRINTADMIN_AUTH "\n"
+			     CUPS_DEFAULT_PRINTOPERATOR_AUTH "\n"
 			     "  </Limit>\n");
       }
 
@@ -1596,7 +1702,8 @@ _cupsAdminSetServerSettings(
 	  cupsFilePrintf(temp, "  Allow %s\n",
 	                 remote_any > 0 ? "all" : "@LOCAL");
       }
-      else if (in_root_location && (remote_admin >= 0 || share_printers >= 0))
+      else if (in_root_location &&
+               (remote_admin >= 0 || remote_any > 0 || share_printers >= 0))
       {
 	wrote_root_location = 1;
 
@@ -1607,12 +1714,14 @@ _cupsAdminSetServerSettings(
           cupsFilePuts(temp, "  # Allow remote administration...\n");
 	else if (share_printers > 0)
           cupsFilePuts(temp, "  # Allow shared printing...\n");
+	else if (remote_any > 0)
+          cupsFilePuts(temp, "  # Allow remote access...\n");
 	else
           cupsFilePuts(temp, "  # Restrict access to the server...\n");
 
         cupsFilePuts(temp, "  Order allow,deny\n");
 
-	if (remote_admin > 0 || share_printers > 0)
+	if (remote_admin > 0 || remote_any > 0 || share_printers > 0)
 	  cupsFilePrintf(temp, "  Allow %s\n",
 	                 remote_any > 0 ? "all" : "@LOCAL");
       }
@@ -1623,84 +1732,86 @@ _cupsAdminSetServerSettings(
 
       cupsFilePuts(temp, "</Location>\n");
     }
-    else if (!strcasecmp(line, "<Limit") && in_default_policy)
+    else if (!strcasecmp(line, "<Limit"))
     {
-     /*
-      * See if the policy limit is for the Cancel-Job operation...
-      */
-
-      char	*valptr;		/* Pointer into value */
-
-
-      indent += 2;
-
-      if (!strcasecmp(value, "cancel-job") && user_cancel_any >= 0)
+      if (in_default_policy)
       {
        /*
-	* Don't write anything for this limit section...
+	* See if the policy limit is for the Cancel-Job operation...
 	*/
-
-	in_cancel_job = 2;
+  
+	char	*valptr;		/* Pointer into value */
+  
+  
+	if (!strcasecmp(value, "cancel-job") && user_cancel_any >= 0)
+	{
+	 /*
+	  * Don't write anything for this limit section...
+	  */
+  
+	  in_cancel_job = 2;
+	}
+	else
+	{
+	  cupsFilePrintf(temp, "%*s%s", indent, "", line);
+  
+	  while (*value)
+	  {
+	    for (valptr = value; *valptr && !_cups_isspace(*valptr); valptr ++);
+  
+	    if (*valptr)
+	      *valptr++ = '\0';
+  
+	    if (!strcasecmp(value, "cancel-job") && user_cancel_any >= 0)
+	    {
+	     /*
+	      * Write everything except for this definition...
+	      */
+  
+	      in_cancel_job = 1;
+	    }
+	    else
+	      cupsFilePrintf(temp, " %s", value);
+  
+	    for (value = valptr; _cups_isspace(*value); value ++);
+	  }
+  
+	  cupsFilePuts(temp, ">\n");
+	}
       }
       else
-      {
-	cupsFilePrintf(temp, "  %s", line);
+        cupsFilePrintf(temp, "%*s%s %s>\n", indent, "", line, value);
 
-	while (*value)
-	{
-	  for (valptr = value; !isspace(*valptr & 255) && *valptr; valptr ++);
-
-	  if (*valptr)
-	    *valptr++ = '\0';
-
-          if (!strcasecmp(value, "cancel-job") && user_cancel_any >= 0)
-	  {
-	   /*
-	    * Write everything except for this definition...
-	    */
-
-	    in_cancel_job = 1;
-	  }
-	  else
-	    cupsFilePrintf(temp, " %s", value);
-
-          for (value = valptr; isspace(*value & 255); value ++);
-	}
-
-	cupsFilePuts(temp, ">\n");
-      }
+      indent += 2;
     }
     else if (!strcasecmp(line, "</Limit>") && in_cancel_job)
     {
       indent -= 2;
 
       if (in_cancel_job == 1)
-        cupsFilePuts(temp, "  </Limit>\n");
+	cupsFilePuts(temp, "  </Limit>\n");
 
       wrote_policy = 1;
 
       if (!user_cancel_any)
 	cupsFilePuts(temp, "  # Only the owner or an administrator can cancel "
-	                   "a job...\n"
-	                   "  <Limit Cancel-Job>\n"
-	                   "    Order deny,allow\n"
-	                   "    Require user @OWNER "
-			   CUPS_DEFAULT_PRINTADMIN_AUTH "\n"
+			   "a job...\n"
+			   "  <Limit Cancel-Job>\n"
+			   "    Order deny,allow\n"
+			   "    Require user @OWNER "
+			   CUPS_DEFAULT_PRINTOPERATOR_AUTH "\n"
 			   "  </Limit>\n");
 
       in_cancel_job = 0;
     }
     else if ((((in_admin_location || in_conf_location || in_root_location) &&
-               remote_admin >= 0) ||
+               (remote_admin >= 0 || remote_any > 0)) ||
               (in_root_location && share_printers >= 0)) &&
-             (((!strcasecmp(line, "Allow") || !strcasecmp(line, "Deny")) &&
-	       !strcasecmp(value, "@LOCAL")) ||
+             (!strcasecmp(line, "Allow") || !strcasecmp(line, "Deny") ||
 	      !strcasecmp(line, "Order")))
       continue;
     else if (in_cancel_job == 2)
       continue;
-    else if (!strcasecmp(line, "<Limit")  && value)
-      cupsFilePrintf(temp, "  %s %s>\n", line, value);
     else if (line[0] == '<')
     {
       if (value)
@@ -1740,7 +1851,7 @@ _cupsAdminSetServerSettings(
        /*
         * Record the non-policy, non-location directives that we find
 	* in the server settings, since we cache this info and record it
-	* in _cupsAdminGetServerSettings()...
+	* in cupsAdminGetServerSettings()...
 	*/
 
 	cupsd_num_settings = cupsAddOption(line, value, cupsd_num_settings,
@@ -1794,13 +1905,14 @@ _cupsAdminSetServerSettings(
     else
     {
       cupsFilePuts(temp, "# Show general information in error_log.\n");
-      cupsFilePuts(temp, "LogLevel info\n");
+      cupsFilePuts(temp, "LogLevel " CUPS_DEFAULT_LOG_LEVEL "\n");
     }
   }
 
-  if (!wrote_port_listen && (share_printers >= 0 || remote_admin >= 0))
+  if (!wrote_port_listen &&
+      (remote_admin >= 0 || remote_any > 0 || share_printers >= 0))
   {
-    if (share_printers > 0 || remote_admin > 0)
+    if (remote_admin > 0 || remote_any > 0 || share_printers > 0)
     {
       cupsFilePuts(temp, "# Allow remote access\n");
       cupsFilePrintf(temp, "Port %d\n", ippPort());
@@ -1818,7 +1930,8 @@ _cupsAdminSetServerSettings(
 #endif /* CUPS_DEFAULT_DOMAINSOCKET */
   }
 
-  if (!wrote_root_location && (remote_admin >= 0 || share_printers >= 0))
+  if (!wrote_root_location &&
+      (remote_admin >= 0 || remote_any > 0 || share_printers >= 0))
   {
     if (remote_admin > 0 && share_printers > 0)
       cupsFilePuts(temp,
@@ -1827,13 +1940,15 @@ _cupsAdminSetServerSettings(
       cupsFilePuts(temp, "# Allow remote administration...\n");
     else if (share_printers > 0)
       cupsFilePuts(temp, "# Allow shared printing...\n");
+    else if (remote_any > 0)
+      cupsFilePuts(temp, "# Allow remote access...\n");
     else
       cupsFilePuts(temp, "# Restrict access to the server...\n");
 
     cupsFilePuts(temp, "<Location />\n"
                        "  Order allow,deny\n");
 
-    if (remote_admin > 0 || share_printers > 0)
+    if (remote_admin > 0 || remote_any > 0 || share_printers > 0)
       cupsFilePrintf(temp, "  Allow %s\n", remote_any > 0 ? "all" : "@LOCAL");
 
     cupsFilePuts(temp, "</Location>\n");
@@ -1911,7 +2026,7 @@ _cupsAdminSetServerSettings(
 	                 "  <Limit Cancel-Job>\n"
 	                 "    Order deny,allow\n"
 	                 "    Require user @OWNER "
-			 CUPS_DEFAULT_PRINTADMIN_AUTH "\n"
+			 CUPS_DEFAULT_PRINTOPERATOR_AUTH "\n"
 			 "  </Limit>\n");
 
     cupsFilePuts(temp, "  <Limit All>\n"
@@ -2061,18 +2176,23 @@ do_samba_command(const char *command,	/* I - Command to run */
     * Child goes here, redirect stdin/out/err and execute the command...
     */
 
-    close(0);
-    open("/dev/null", O_RDONLY);
+    int fd = open("/dev/null", O_RDONLY);
 
-    close(1);
+    if (fd > 0)
+    {
+      dup2(fd, 0);
+      close(fd);
+    }
 
     if (logfile)
-      dup(fileno(logfile));
-    else
-      open("/dev/null", O_WRONLY);
+      dup2(fileno(logfile), 1);
+    else if ((fd = open("/dev/null", O_WRONLY)) > 1)
+    {
+      dup2(fd, 1);
+      close(fd);
+    }
 
-    close(2);
-    dup(1);
+    dup2(1, 2);
 
     execlp(command, command, address, "-N", "-A", authfile, "-c", subcmd,
            (char *)0);
@@ -2098,7 +2218,7 @@ do_samba_command(const char *command,	/* I - Command to run */
   if (logfile)
     _cupsLangPuts(logfile, "\n");
 
-  DEBUG_printf(("status=%d\n", status));
+  DEBUG_printf(("9do_samba_command: status=%d", status));
 
   if (WIFEXITED(status))
     return (WEXITSTATUS(status));
@@ -2156,7 +2276,7 @@ get_cupsd_conf(
       snprintf(message, sizeof(message),
                _cupsLangString(cupsLangDefault(), _("stat of %s failed: %s")),
                name, strerror(errno));
-      _cupsSetError(IPP_INTERNAL_ERROR, message);
+      _cupsSetError(IPP_INTERNAL_ERROR, message, 0);
 
       *name = '\0';
 
@@ -2178,7 +2298,7 @@ get_cupsd_conf(
     {
       *name = '\0';
 
-      _cupsSetError(IPP_INTERNAL_ERROR, strerror(errno));
+      _cupsSetError(IPP_INTERNAL_ERROR, NULL, 0);
 
       invalidate_cupsd_cache(cg);
 
@@ -2335,5 +2455,5 @@ write_option(cups_file_t     *dstfp,	/* I - PPD file */
 
 
 /*
- * End of "$Id: adminutil.c 7264 2008-01-28 23:19:27Z mike $".
+ * End of "$Id: adminutil.c 9376 2010-11-17 19:58:22Z mike $".
  */
