@@ -1,9 +1,9 @@
 /*
- * "$Id: array.c 6649 2007-07-11 21:46:42Z mike $"
+ * "$Id: array.c 8532 2009-04-20 21:37:14Z mike $"
  *
  *   Sorted array routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -99,15 +99,17 @@ static int	cups_array_find(cups_array_t *a, void *e, int prev, int *rdiff);
  * 'cupsArrayAdd()' - Add an element to the array.
  *
  * When adding an element to a sorted array, non-unique elements are
- * appended at the end of the run.  For unsorted arrays, the element
- * is inserted at the end of the array.
+ * appended at the end of the run of identical elements.  For unsorted arrays,
+ * the element is appended to the end of the array.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 int					/* O - 1 on success, 0 on failure */
 cupsArrayAdd(cups_array_t *a,		/* I - Array */
              void         *e)		/* I - Element */
 {
-  DEBUG_printf(("cupsArrayAdd(a=%p, e=%p)\n", a, e));
+  DEBUG_printf(("2cupsArrayAdd(a=%p, e=%p)", a, e));
 
  /*
   * Range check input...
@@ -115,7 +117,7 @@ cupsArrayAdd(cups_array_t *a,		/* I - Array */
 
   if (!a || !e)
   {
-    DEBUG_puts("cupsArrayAdd: returning 0");
+    DEBUG_puts("3cupsArrayAdd: returning 0");
     return (0);
   }
 
@@ -129,6 +131,12 @@ cupsArrayAdd(cups_array_t *a,		/* I - Array */
 
 /*
  * 'cupsArrayClear()' - Clear the array.
+ *
+ * This function is equivalent to removing all elements in the array.
+ * The caller is responsible for freeing the memory used by the
+ * elements themselves.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 void
@@ -156,6 +164,8 @@ cupsArrayClear(cups_array_t *a)		/* I - Array */
 
 /*
  * 'cupsArrayCount()' - Get the number of elements in the array.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 int					/* O - Number of elements */
@@ -178,6 +188,11 @@ cupsArrayCount(cups_array_t *a)		/* I - Array */
 
 /*
  * 'cupsArrayCurrent()' - Return the current element in the array.
+ *
+ * The current element is undefined until you call @link cupsArrayFind@,
+ * @link cupsArrayFirst@, or @link cupsArrayIndex@, or @link cupsArrayLast@.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 void *					/* O - Element */
@@ -203,6 +218,11 @@ cupsArrayCurrent(cups_array_t *a)	/* I - Array */
 
 /*
  * 'cupsArrayDelete()' - Free all memory used by the array.
+ *
+ * The caller is responsible for freeing the memory used by the
+ * elements themselves.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 void
@@ -232,6 +252,8 @@ cupsArrayDelete(cups_array_t *a)	/* I - Array */
 
 /*
  * 'cupsArrayDup()' - Duplicate the array.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 cups_array_t *				/* O - Duplicate array */
@@ -296,9 +318,11 @@ cupsArrayDup(cups_array_t *a)		/* I - Array */
 
 /*
  * 'cupsArrayFind()' - Find an element in the array.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
-void *					/* O - Element found or NULL */
+void *					/* O - Element found or @code NULL@ */
 cupsArrayFind(cups_array_t *a,		/* I - Array */
               void         *e)		/* I - Element */
 {
@@ -389,9 +413,11 @@ cupsArrayFind(cups_array_t *a,		/* I - Array */
 
 /*
  * 'cupsArrayFirst()' - Get the first element in the array.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
-void *					/* O - First element or NULL */
+void *					/* O - First element or @code NULL@ if the array is empty */
 cupsArrayFirst(cups_array_t *a)		/* I - Array */
 {
  /*
@@ -414,10 +440,13 @@ cupsArrayFirst(cups_array_t *a)		/* I - Array */
 /*
  * 'cupsArrayGetIndex()' - Get the index of the current element.
  *
- * @since CUPS 1.3@
+ * The current element is undefined until you call @link cupsArrayFind@,
+ * @link cupsArrayFirst@, or @link cupsArrayIndex@, or @link cupsArrayLast@.
+ *
+ * @since CUPS 1.3/Mac OS X 10.5@
  */
 
-int					/* O - Index of the current element */
+int					/* O - Index of the current element, starting at 0 */
 cupsArrayGetIndex(cups_array_t *a)	/* I - Array */
 {
   if (!a)
@@ -430,10 +459,10 @@ cupsArrayGetIndex(cups_array_t *a)	/* I - Array */
 /*
  * 'cupsArrayGetInsert()' - Get the index of the last inserted element.
  *
- * @since CUPS 1.3@
+ * @since CUPS 1.3/Mac OS X 10.5@
  */
 
-int					/* O - Index of the last inserted element */
+int					/* O - Index of the last inserted element, starting at 0 */
 cupsArrayGetInsert(cups_array_t *a)	/* I - Array */
 {
   if (!a)
@@ -445,9 +474,11 @@ cupsArrayGetInsert(cups_array_t *a)	/* I - Array */
 
 /*
  * 'cupsArrayIndex()' - Get the N-th element in the array.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
-void *					/* O - N-th element or NULL */
+void *					/* O - N-th element or @code NULL@ */
 cupsArrayIndex(cups_array_t *a,		/* I - Array */
                int          n)		/* I - Index into array, starting at 0 */
 {
@@ -464,15 +495,17 @@ cupsArrayIndex(cups_array_t *a,		/* I - Array */
  * 'cupsArrayInsert()' - Insert an element in the array.
  *
  * When inserting an element in a sorted array, non-unique elements are
- * inserted at the beginning of the run.  For unsorted arrays, the element
- * is inserted at the beginning of the array.
+ * inserted at the beginning of the run of identical elements.  For unsorted
+ * arrays, the element is inserted at the beginning of the array.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 int					/* O - 0 on failure, 1 on success */
 cupsArrayInsert(cups_array_t *a,	/* I - Array */
 		void         *e)	/* I - Element */
 {
-  DEBUG_printf(("cupsArrayInsert(a=%p, e=%p)\n", a, e));
+  DEBUG_printf(("2cupsArrayInsert(a=%p, e=%p)", a, e));
 
  /*
   * Range check input...
@@ -480,7 +513,7 @@ cupsArrayInsert(cups_array_t *a,	/* I - Array */
 
   if (!a || !e)
   {
-    DEBUG_puts("cupsArrayInsert: returning 0");
+    DEBUG_puts("3cupsArrayInsert: returning 0");
     return (0);
   }
 
@@ -494,9 +527,11 @@ cupsArrayInsert(cups_array_t *a,	/* I - Array */
 
 /*
  * 'cupsArrayLast()' - Get the last element in the array.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
-void *					/* O - Last element or NULL */
+void *					/* O - Last element or @code NULL@ if the array is empty */
 cupsArrayLast(cups_array_t *a)		/* I - Array */
 {
  /*
@@ -518,11 +553,18 @@ cupsArrayLast(cups_array_t *a)		/* I - Array */
 
 /*
  * 'cupsArrayNew()' - Create a new array.
+ *
+ * The comparison function ("f") is used to create a sorted array. The function
+ * receives pointers to two elements and the user data pointer ("d") - the user
+ * data pointer argument can safely be omitted when not required so functions
+ * like @code strcmp@ can be used for sorted string arrays.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 cups_array_t *				/* O - Array */
-cupsArrayNew(cups_array_func_t f,	/* I - Comparison function */
-             void              *d)	/* I - User data */
+cupsArrayNew(cups_array_func_t f,	/* I - Comparison function or @code NULL@ for an unsorted array */
+             void              *d)	/* I - User data pointer or @code NULL@ */
 {
   return (cupsArrayNew2(f, d, 0, 0));
 }
@@ -531,14 +573,22 @@ cupsArrayNew(cups_array_func_t f,	/* I - Comparison function */
 /*
  * 'cupsArrayNew2()' - Create a new array with hash.
  *
- * @since CUPS 1.3@
+ * The comparison function ("f") is used to create a sorted array. The function
+ * receives pointers to two elements and the user data pointer ("d") - the user
+ * data pointer argument can safely be omitted when not required so functions
+ * like @code strcmp@ can be used for sorted string arrays.
+ *
+ * The hash function ("h") is used to implement cached lookups with the
+ * specified hash size ("hsize").
+ *
+ * @since CUPS 1.3/Mac OS X 10.5@
  */
 
 cups_array_t *				/* O - Array */
-cupsArrayNew2(cups_array_func_t  f,	/* I - Comparison function */
-              void               *d,	/* I - User data */
-              cups_ahash_func_t  h,	/* I - Hash function*/
-	      int                hsize)	/* I - Hash size */
+cupsArrayNew2(cups_array_func_t  f,	/* I - Comparison function or @code NULL@ for an unsorted array */
+              void               *d,	/* I - User data or @code NULL@ */
+              cups_ahash_func_t  h,	/* I - Hash function or @code NULL@ for unhashed lookups */
+	      int                hsize)	/* I - Hash size (>= 0) */
 {
   cups_array_t	*a;			/* Array  */
 
@@ -579,9 +629,17 @@ cupsArrayNew2(cups_array_func_t  f,	/* I - Comparison function */
 
 /*
  * 'cupsArrayNext()' - Get the next element in the array.
+ *
+ * This function is equivalent to "cupsArrayIndex(a, cupsArrayGetIndex(a) + 1)".
+ *
+ * The next element is undefined until you call @link cupsArrayFind@,
+ * @link cupsArrayFirst@, or @link cupsArrayIndex@, or @link cupsArrayLast@
+ * to set the current element.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
-void *					/* O - Next element or NULL */
+void *					/* O - Next element or @code NULL@ */
 cupsArrayNext(cups_array_t *a)		/* I - Array */
 {
  /*
@@ -604,9 +662,17 @@ cupsArrayNext(cups_array_t *a)		/* I - Array */
 
 /*
  * 'cupsArrayPrev()' - Get the previous element in the array.
+ *
+ * This function is equivalent to "cupsArrayIndex(a, cupsArrayGetIndex(a) - 1)".
+ *
+ * The previous element is undefined until you call @link cupsArrayFind@,
+ * @link cupsArrayFirst@, or @link cupsArrayIndex@, or @link cupsArrayLast@
+ * to set the current element.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
-void *					/* O - Previous element or NULL */
+void *					/* O - Previous element or @code NULL@ */
 cupsArrayPrev(cups_array_t *a)		/* I - Array */
 {
  /*
@@ -629,6 +695,14 @@ cupsArrayPrev(cups_array_t *a)		/* I - Array */
 
 /*
  * 'cupsArrayRemove()' - Remove an element from the array.
+ *
+ * If more than one element matches "e", only the first matching element is
+ * removed.
+ *
+ * The caller is responsible for freeing the memory used by the
+ * removed element.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 int					/* O - 1 on success, 0 on failure */
@@ -688,7 +762,9 @@ cupsArrayRemove(cups_array_t *a,	/* I - Array */
 
 
 /*
- * 'cupsArrayRestore()' - Reset the current element to the last cupsArraySave.
+ * 'cupsArrayRestore()' - Reset the current element to the last @link cupsArraySave@.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 void *					/* O - New current element */
@@ -711,9 +787,15 @@ cupsArrayRestore(cups_array_t *a)	/* I - Array */
 
 
 /*
- * 'cupsArraySave()' - Mark the current element for a later cupsArrayRestore.
+ * 'cupsArraySave()' - Mark the current element for a later @link cupsArrayRestore@.
+ *
+ * The current element is undefined until you call @link cupsArrayFind@,
+ * @link cupsArrayFirst@, or @link cupsArrayIndex@, or @link cupsArrayLast@
+ * to set the current element.
  *
  * The save/restore stack is guaranteed to be at least 32 elements deep.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 int					/* O - 1 on success, 0 on failure */
@@ -734,6 +816,8 @@ cupsArraySave(cups_array_t *a)		/* I - Array */
 
 /*
  * 'cupsArrayUserData()' - Return the user data for an array.
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 void *					/* O - User data */
@@ -748,6 +832,8 @@ cupsArrayUserData(cups_array_t *a)	/* I - Array */
 
 /*
  * 'cups_array_add()' - Insert or append an element to the array...
+ *
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 static int				/* O - 1 on success, 0 on failure */
@@ -760,7 +846,7 @@ cups_array_add(cups_array_t *a,		/* I - Array */
 	diff;				/* Comparison with current element */
 
 
-  DEBUG_printf(("cups_array_add(a=%p, e=%p, insert=%d)\n", a, e, insert));
+  DEBUG_printf(("7cups_array_add(a=%p, e=%p, insert=%d)", a, e, insert));
 
  /*
   * Verify we have room for the new element...
@@ -793,11 +879,11 @@ cups_array_add(cups_array_t *a,		/* I - Array */
       temp = realloc(a->elements, count * sizeof(void *));
     }
 
-    DEBUG_printf(("cups_array_add: count=%d\n", count));
+    DEBUG_printf(("9cups_array_add: count=%d", count));
 
     if (!temp)
     {
-      DEBUG_puts("cupsAddAdd: allocation failed, returning 0");
+      DEBUG_puts("9cups_array_add: allocation failed, returning 0");
       return (0);
     }
 
@@ -892,11 +978,11 @@ cups_array_add(cups_array_t *a,		/* I - Array */
       if (a->saved[i] >= current)
 	a->saved[i] ++;
 
-    DEBUG_printf(("cups_array_add: insert element at index %d...\n", current));
+    DEBUG_printf(("9cups_array_add: insert element at index %d...", current));
   }
 #ifdef DEBUG
   else
-    printf("cups_array_add: append element at %d...\n", current);
+    DEBUG_printf(("9cups_array_add: append element at %d...", current));
 #endif /* DEBUG */
 
   a->elements[current] = e;
@@ -905,10 +991,11 @@ cups_array_add(cups_array_t *a,		/* I - Array */
 
 #ifdef DEBUG
   for (current = 0; current < a->num_elements; current ++)
-    printf("cups_array_add: a->elements[%d]=%p\n", current, a->elements[current]);
+    DEBUG_printf(("9cups_array_add: a->elements[%d]=%p", current,
+                  a->elements[current]));
 #endif /* DEBUG */
 
-  DEBUG_puts("cups_array_add: returning 1");
+  DEBUG_puts("9cups_array_add: returning 1");
 
   return (1);
 }
@@ -930,7 +1017,7 @@ cups_array_find(cups_array_t *a,	/* I - Array */
 	diff;				/* Comparison with current element */
 
 
-  DEBUG_printf(("cups_array_find(a=%p, e=%p, prev=%d, rdiff=%p)\n", a, e, prev,
+  DEBUG_printf(("7cups_array_find(a=%p, e=%p, prev=%d, rdiff=%p)", a, e, prev,
                 rdiff));
 
   if (a->compare)
@@ -939,7 +1026,7 @@ cups_array_find(cups_array_t *a,	/* I - Array */
     * Do a binary search for the element...
     */
 
-    DEBUG_puts("cups_array_find: binary search");
+    DEBUG_puts("9cups_array_find: binary search");
 
     if (prev >= 0 && prev < a->num_elements)
     {
@@ -955,7 +1042,7 @@ cups_array_find(cups_array_t *a,	/* I - Array */
         * Exact or edge match, return it!
 	*/
 
-        DEBUG_printf(("cups_array_find: Returning %d, diff=%d\n", prev, diff));
+        DEBUG_printf(("9cups_array_find: Returning %d, diff=%d", prev, diff));
 
 	*rdiff = diff;
 
@@ -995,7 +1082,7 @@ cups_array_find(cups_array_t *a,	/* I - Array */
       current = (left + right) / 2;
       diff    = (*(a->compare))(e, a->elements[current], a->data);
 
-      DEBUG_printf(("cups_array_find: left=%d, right=%d, current=%d, diff=%d\n",
+      DEBUG_printf(("9cups_array_find: left=%d, right=%d, current=%d, diff=%d",
                     left, right, current, diff));
 
       if (diff == 0)
@@ -1028,7 +1115,7 @@ cups_array_find(cups_array_t *a,	/* I - Array */
     * Do a linear pointer search...
     */
 
-    DEBUG_puts("cups_array_find: linear search");
+    DEBUG_puts("9cups_array_find: linear search");
 
     diff = 1;
 
@@ -1044,7 +1131,7 @@ cups_array_find(cups_array_t *a,	/* I - Array */
   * Return the closest element and the difference...
   */
 
-  DEBUG_printf(("cups_array_find: Returning %d, diff=%d\n", current, diff));
+  DEBUG_printf(("8cups_array_find: Returning %d, diff=%d", current, diff));
 
   *rdiff = diff;
 
@@ -1053,5 +1140,5 @@ cups_array_find(cups_array_t *a,	/* I - Array */
 
 
 /*
- * End of "$Id: array.c 6649 2007-07-11 21:46:42Z mike $".
+ * End of "$Id: array.c 8532 2009-04-20 21:37:14Z mike $".
  */
