@@ -1,25 +1,16 @@
 /*
- * "$Id: imagetops.c 5379 2006-04-07 13:48:37Z mike $"
+ * "$Id: imagetops.c 8152 2008-11-21 19:00:11Z mike $"
  *
  *   Image file to PostScript filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2006 by Easy Software Products.
+ *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -37,6 +28,7 @@
 #include "common.h"
 #include "image.h"
 #include <math.h>
+#include <cups/i18n.h>
 
 
 /*
@@ -120,13 +112,11 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (argc < 6 || argc > 7)
   {
-    fputs("ERROR: imagetops job-id user title copies options [file]\n", stderr);
+    fprintf(stderr, _("Usage: %s job-id user title copies options [file]\n"),
+            argv[0]);
     return (1);
   }
 
-  fprintf(stderr, "INFO: %s %s %s %s %s %s %s\n", argv[0], argv[1], argv[2],
-          argv[3], argv[4], argv[5], argv[6] ? argv[6] : "(null)");
-
  /*
   * Copy stdin as needed...
   */
@@ -222,7 +212,11 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if ((val = cupsGetOption("scaling", num_options, options)) != NULL)
     zoom = atoi(val) * 0.01;
-  else if (cupsGetOption("fitplot", num_options, options))
+  else if ((val = cupsGetOption("fitplot", num_options, options)) != NULL &&
+           !strcasecmp(val, "true"))
+    zoom = 1.0;
+  else if ((val = cupsGetOption("fit-to-page", num_options, options)) != NULL &&
+           !strcasecmp(val, "true"))
     zoom = 1.0;
 
   if ((val = cupsGetOption("ppi", num_options, options)) != NULL)
@@ -284,8 +278,16 @@ main(int  argc,				/* I - Number of command-line arguments */
   if ((val = cupsGetOption("hue", num_options, options)) != NULL)
     hue = atoi(val);
 
-  if ((val = cupsGetOption("mirror", num_options, options)) != NULL &&
-      strcasecmp(val, "True") == 0)
+  if ((choice = ppdFindMarkedChoice(ppd, "MirrorPrint")) != NULL)
+  {
+    val = choice->choice;
+    choice->marked = 0;
+  }
+  else
+    val = cupsGetOption("mirror", num_options, options);
+
+  if (val && (!strcasecmp(val, "true") || !strcasecmp(val, "on") ||
+              !strcasecmp(val, "yes")))
     Flip = 1;
 
   if ((val = cupsGetOption("emit-jcl", num_options, options)) != NULL &&
@@ -308,7 +310,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (img == NULL)
   {
-    fputs("ERROR: Unable to open image file for printing!\n", stderr);
+    fputs(_("ERROR: Unable to open image file for printing!\n"), stderr);
     ppdClose(ppd);
     return (1);
   }
@@ -609,8 +611,8 @@ main(int  argc,				/* I - Number of command-line arguments */
   puts("%%Creator: imagetops/" CUPS_SVERSION);
   strftime(curdate, sizeof(curdate), "%c", curtm);
   printf("%%%%CreationDate: %s\n", curdate);
-  printf("%%%%Title: %s\n", argv[3]);
-  printf("%%%%For: %s\n", argv[2]);
+  WriteTextComment("Title", argv[3]);
+  WriteTextComment("For", argv[2]);
   if (Orientation & 1)
     puts("%%Orientation: Landscape");
   else
@@ -803,7 +805,7 @@ main(int  argc,				/* I - Number of command-line arguments */
         if (ppd && ppd->num_filters == 0)
           fprintf(stderr, "PAGE: %d %d\n", page, realcopies);
 
-	fprintf(stderr, "INFO: Printing page %d...\n", page);
+	fprintf(stderr, _("INFO: Printing page %d...\n"), page);
 
         printf("%%%%Page: %d %d\n", page, page);
 
@@ -1063,5 +1065,5 @@ ps_ascii85(cups_ib_t *data,		/* I - Data to print */
 
 
 /*
- * End of "$Id: imagetops.c 5379 2006-04-07 13:48:37Z mike $".
+ * End of "$Id: imagetops.c 8152 2008-11-21 19:00:11Z mike $".
  */
