@@ -1,9 +1,9 @@
 /*
- * "$Id: mailto.c 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: mailto.c 9198 2010-07-08 17:24:14Z mike $"
  *
  *   "mailto" notifier for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -315,7 +315,7 @@ email_message(const char *to,		/* I - Recipient of message */
   cupsFilePrintf(fp, "Content-Type: text/plain%s", nl);
   cupsFilePuts(fp, nl);
   cupsFilePrintf(fp, "%s%s", text, nl);
-  cupsFilePrintf(fp, ".\n", nl);
+  cupsFilePrintf(fp, ".%s", nl);
 
  /*
   * Close the connection to the mail server...
@@ -369,8 +369,10 @@ email_message(const char *to,		/* I - Recipient of message */
     fputs("DEBUG: QUIT\n", stderr);
 
     if (!cupsFileGets(fp, response, sizeof(response)) || atoi(response) >= 500)
-      goto smtp_error;
-    fprintf(stderr, "DEBUG: <<< %s\n", response);
+      fprintf(stderr, "ERROR: Got \"%s\" trying to QUIT connection.\n",
+              response);
+    else
+      fprintf(stderr, "DEBUG: <<< %s\n", response);
 
     cupsFileClose(fp);
 
@@ -786,5 +788,5 @@ print_attributes(ipp_t *ipp,		/* I - IPP request */
 
 
 /*
- * End of "$Id: mailto.c 7721 2008-07-11 22:48:49Z mike $".
+ * End of "$Id: mailto.c 9198 2010-07-08 17:24:14Z mike $".
  */
