@@ -1,5 +1,5 @@
 /*
- * "$Id: printers.h 8727 2009-06-25 17:07:26Z mike $"
+ * "$Id: printers.h 9061 2010-03-30 22:07:33Z mike $"
  *
  *   Printer definitions for the Common UNIX Printing System (CUPS) scheduler.
  *
@@ -16,6 +16,8 @@
 #ifdef HAVE_DNSSD
 #  include <dns_sd.h>
 #endif /* HAVE_DNSSD */
+#include <cups/pwg-private.h>
+
 
 /*
  * Quota data...
@@ -95,6 +97,7 @@ typedef struct cupsd_printer_s
   cups_array_t	*filters,		/* Filters for queue */
 		*pre_filters;		/* Pre-filters for queue */
   char		*recoverable;		/* com.apple.print.recoverable-message */
+  _pwg_t	*pwg;			/* PWG<->PPD mapping data */
 
 #ifdef HAVE_DNSSD
   char		*reg_name,		/* Name used for service registration */
@@ -141,7 +144,7 @@ extern void		cupsdAddPrinterUser(cupsd_printer_t *p,
 			                    const char *username);
 extern void		cupsdCreateCommonData(void);
 extern void		cupsdDeleteAllPrinters(void);
-extern void		cupsdDeletePrinter(cupsd_printer_t *p, int update);
+extern int		cupsdDeletePrinter(cupsd_printer_t *p, int update);
 extern cupsd_printer_t	*cupsdFindDest(const char *name);
 extern cupsd_printer_t	*cupsdFindPrinter(const char *name);
 extern cupsd_quota_t	*cupsdFindQuota(cupsd_printer_t *p,
@@ -179,5 +182,5 @@ extern void		cupsdWritePrintcap(void);
 
 
 /*
- * End of "$Id: printers.h 8727 2009-06-25 17:07:26Z mike $".
+ * End of "$Id: printers.h 9061 2010-03-30 22:07:33Z mike $".
  */
