@@ -1,9 +1,9 @@
 /*
- * "$Id: cupstestppd.c 8835 2009-10-06 18:50:58Z mike $"
+ * "$Id: cupstestppd.c 9125 2010-04-26 18:24:37Z mike $"
  *
- *   PPD test program for the Common UNIX Printing System (CUPS).
+ *   PPD test program for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -69,7 +69,8 @@ enum
   WARN_TRANSLATIONS = 16,
   WARN_DUPLEX = 32,
   WARN_SIZES = 64,
-  WARN_ALL = 127
+  WARN_FILENAME = 128,
+  WARN_ALL = 255
 };
 
 
@@ -104,175 +105,13 @@ enum
  * File permissions...
  */
 
-#define MODE_WRITE	0002		/* Other write */
+#define MODE_WRITE	0022		/* Group/other write */
 #define MODE_MASK	0555		/* Owner/group/other read+exec/search */
 #define MODE_DATAFILE	0444		/* Owner/group/other read */
 #define MODE_DIRECTORY	0555		/* Owner/group/other read+search */
 #define MODE_PROGRAM	0555		/* Owner/group/other read+exec */
 
 
-/*
- * Standard Adobe media keywords (must remain sorted)...
- */
-
-static const char adobe_size_names[][PPD_MAX_NAME] =
-{
-  "10x11",
-  "10x13",
-  "10x14",
-  "12x11",
-  "15x11",
-  "7x9",
-  "8x10",
-  "9x11",
-  "9x12",
-  "A0",
-  "A1",
-  "A10",
-  "A2",
-  "A3",
-  "A3Extra",
-  "A3Rotated",
-  "A4",
-  "A4Extra",
-  "A4Plus",
-  "A4Rotated",
-  "A4Small",
-  "A5",
-  "A5Extra",
-  "A5Rotated",
-  "A6",
-  "A6Rotated",
-  "A7",
-  "A8",
-  "A9",
-  "ARCHA",
-  "ARCHB",
-  "ARCHC",
-  "ARCHD",
-  "ARCHE",
-  "AnsiA",
-  "AnsiB",
-  "AnsiC",
-  "AnsiD",
-  "AnsiE",
-  "B0",
-  "B1",
-  "B1",
-  "B10",
-  "B2",
-  "B3",
-  "B4",
-  "B4Rotated",
-  "B5",
-  "B5Rotated",
-  "B6",
-  "B6Rotated",
-  "B7",
-  "B8",
-  "B9",
-  "C4",
-  "C5",
-  "C6",
-  "DL",
-  "DoublePostcard",
-  "DoublePostcardRotated",
-  "Env10",
-  "Env11",
-  "Env12",
-  "Env14",
-  "Env9",
-  "EnvC0",
-  "EnvC1",
-  "EnvC2",
-  "EnvC3",
-  "EnvC4",
-  "EnvC5",
-  "EnvC6",
-  "EnvC65",
-  "EnvC7",
-  "EnvChou3",
-  "EnvChou3Rotated",
-  "EnvChou4",
-  "EnvChou4Rotated",
-  "EnvDL",
-  "EnvISOB4",
-  "EnvISOB5",
-  "EnvISOB6",
-  "EnvInvite",
-  "EnvItalian",
-  "EnvKaku2",
-  "EnvKaku2Rotated",
-  "EnvKaku3",
-  "EnvKaku3Rotated",
-  "EnvMonarch",
-  "EnvPRC1",
-  "EnvPRC10",
-  "EnvPRC10Rotated",
-  "EnvPRC1Rotated",
-  "EnvPRC2",
-  "EnvPRC2Rotated",
-  "EnvPRC3",
-  "EnvPRC3Rotated",
-  "EnvPRC4",
-  "EnvPRC4Rotated",
-  "EnvPRC5",
-  "EnvPRC5Rotated",
-  "EnvPRC6",
-  "EnvPRC6Rotated",
-  "EnvPRC7",
-  "EnvPRC7Rotated",
-  "EnvPRC8",
-  "EnvPRC8Rotated",
-  "EnvPRC9",
-  "EnvPRC9Rotated",
-  "EnvPersonal",
-  "EnvYou4",
-  "EnvYou4Rotated",
-  "Executive",
-  "FanFoldGerman",
-  "FanFoldGermanLegal",
-  "FanFoldUS",
-  "Folio",
-  "ISOB0",
-  "ISOB1",
-  "ISOB10",
-  "ISOB2",
-  "ISOB3",
-  "ISOB4",
-  "ISOB5",
-  "ISOB5Extra",
-  "ISOB6",
-  "ISOB7",
-  "ISOB8",
-  "ISOB9",
-  "Ledger",
-  "Legal",
-  "LegalExtra",
-  "Letter",
-  "LetterExtra",
-  "LetterPlus",
-  "LetterRotated",
-  "LetterSmall",
-  "Monarch",
-  "Note",
-  "PRC16K",
-  "PRC16KRotated",
-  "PRC32K",
-  "PRC32KBig",
-  "PRC32KBigRotated",
-  "PRC32KRotated",
-  "Postcard",
-  "PostcardRotated",
-  "Quarto",
-  "Statement",
-  "SuperA",
-  "SuperB",
-  "Tabloid",
-  "TabloidExtra"
-};
-
-
 /*
  * Local functions...
  */
@@ -315,6 +154,7 @@ main(int  argc,				/* I - Number of command-line args */
   int		files;			/* Number of files */
   int		verbose;		/* Want verbose output? */
   int		warn;			/* Which errors to just warn about */
+  int		ignore;			/* Which errors to ignore */
   int		status;			/* Exit status */
   int		errors;			/* Number of conformance errors */
   int		ppdversion;		/* PPD spec version in PPD file */
@@ -352,6 +192,7 @@ main(int  argc,				/* I - Number of command-line args */
   status  = ERROR_NONE;
   root    = "";
   warn    = WARN_NONE;
+  ignore  = WARN_NONE;
 
   for (i = 1; i < argc; i ++)
     if (argv[i][0] == '-' && argv[i][1])
@@ -359,6 +200,26 @@ main(int  argc,				/* I - Number of command-line args */
       for (opt = argv[i] + 1; *opt; opt ++)
         switch (*opt)
 	{
+	  case 'I' :			/* Ignore errors */
+	      i ++;
+
+	      if (i >= argc)
+	        usage();
+
+              if (!strcmp(argv[i], "none"))
+	        ignore = WARN_NONE;
+	      else if (!strcmp(argv[i], "filename"))
+	        ignore |= WARN_FILENAME;
+	      else if (!strcmp(argv[i], "filters"))
+	        ignore |= WARN_FILTERS;
+	      else if (!strcmp(argv[i], "profiles"))
+	        ignore |= WARN_PROFILES;
+	      else if (!strcmp(argv[i], "all"))
+	        ignore = WARN_FILTERS | WARN_PROFILES;
+	      else
+	        usage();
+	      break;
+
 	  case 'R' :			/* Alternate root directory */
 	      i ++;
 
@@ -979,7 +840,7 @@ main(int  argc,				/* I - Number of command-line args */
 	if (verbose > 0)
           _cupsLangPuts(stdout, _("        PASS    PCFileName\n"));
       }
-      else
+      else if (!(ignore & WARN_FILENAME))
       {
 	if (verbose >= 0)
 	{
@@ -1267,10 +1128,10 @@ main(int  argc,				/* I - Number of command-line args */
       if (!(warn & WARN_CONSTRAINTS))
         errors = check_constraints(ppd, errors, verbose, 0);
 
-      if (!(warn & WARN_FILTERS))
+      if (!(warn & WARN_FILTERS) && !(ignore & WARN_FILTERS))
         errors = check_filters(ppd, root, errors, verbose, 0);
 
-      if (!(warn & WARN_PROFILES))
+      if (!(warn & WARN_PROFILES) && !(ignore & WARN_PROFILES))
         errors = check_profiles(ppd, root, errors, verbose, 0);
 
       if (!(warn & WARN_SIZES))
@@ -1404,10 +1265,10 @@ main(int  argc,				/* I - Number of command-line args */
 	if (warn & WARN_CONSTRAINTS)
 	  errors = check_constraints(ppd, errors, verbose, 1);
 
-	if (warn & WARN_FILTERS)
+	if ((warn & WARN_FILTERS) && !(ignore & WARN_FILTERS))
 	  errors = check_filters(ppd, root, errors, verbose, 1);
 
-	if (warn & WARN_PROFILES)
+	if ((warn & WARN_PROFILES) && !(ignore & WARN_PROFILES))
 	  errors = check_profiles(ppd, root, errors, verbose, 1);
 
         if (warn & WARN_SIZES)
@@ -1503,12 +1364,23 @@ main(int  argc,				/* I - Number of command-line args */
 	* a warning and not a hard error...
 	*/
 
-	if (ppd->pcfilename && strlen(ppd->pcfilename) > 12)
-	{
-	  _cupsLangPuts(stdout,
-	                _("        WARN    PCFileName longer than 8.3 in "
-			  "violation of PPD spec.\n"
-			  "                REF: Pages 61-62, section 5.3.\n"));
+        if (!(ignore & WARN_FILENAME) && ppd->pcfilename)
+        {
+	  if (strlen(ppd->pcfilename) > 12)
+	  {
+	    _cupsLangPuts(stdout,
+			  _("        WARN    PCFileName longer than 8.3 in "
+			    "violation of PPD spec.\n"
+			    "                REF: Pages 61-62, section "
+			    "5.3.\n"));
+	  }
+
+	  if (!strcasecmp(ppd->pcfilename, "unused.ppd"))
+	    _cupsLangPuts(stdout,
+	                  _("        WARN    PCFileName should contain a "
+	                    "unique filename.\n"
+			    "                REF: Pages 61-62, section "
+			    "5.3.\n"));
         }
 
         if (!ppd->shortnickname && ppdversion < 43)
@@ -3039,8 +2911,6 @@ check_sizes(ppd_file_t *ppd,		/* I - PPD file */
   ppd_size_t	*size;			/* Current size */
   int		width,			/* Custom width */
 		length;			/* Custom length */
-  char		name[PPD_MAX_NAME],	/* Size name without dot suffix */
-		*nameptr;		/* Pointer into name */
   const char	*prefix;		/* WARN/FAIL prefix */
   ppd_option_t	*page_size,		/* PageSize option */
 		*page_region;		/* PageRegion option */
@@ -3116,28 +2986,6 @@ check_sizes(ppd_file_t *ppd,		/* I - PPD file */
 	  errors ++;
       }
     }
-    else if (warn && verbose >= 0)
-    {
-     /*
-      * Lookup the size name in the standard size table...
-      */
-
-      strlcpy(name, size->name, sizeof(name));
-      if ((nameptr = strchr(name, '.')) != NULL)
-        *nameptr = '\0';
-
-      if (!bsearch(name, adobe_size_names,
-                   sizeof(adobe_size_names) /
-		       sizeof(adobe_size_names[0]),
-		   sizeof(adobe_size_names[0]),
-		   (int (*)(const void *, const void *))strcmp))
-      {
-	_cupsLangPrintf(stdout,
-			_("      %s  Non-standard size name \"%s\"!\n"
-			  "                REF: Page 187, section B.2.\n"),
-			prefix, size->name);
-      }
-    }
 
    /*
     * Verify that the size is defined for both PageSize and PageRegion...
@@ -3806,5 +3654,5 @@ valid_utf8(const char *s)		/* I - String to check */
 
 
 /*
- * End of "$Id: cupstestppd.c 8835 2009-10-06 18:50:58Z mike $".
+ * End of "$Id: cupstestppd.c 9125 2010-04-26 18:24:37Z mike $".
  */
