@@ -1,9 +1,9 @@
 /*
- * "$Id: lp.c 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: lp.c 8141 2008-11-18 00:42:34Z mike $"
  *
  *   "lp" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -17,7 +17,6 @@
  *   main()          - Parse options and send files for printing.
  *   restart_job()   - Restart a job.
  *   set_job_attrs() - Set job attributes.
- *   sighandler()    - Signal catcher for when we print from stdin...
  */
 
 /*
@@ -32,29 +31,15 @@
 #include <cups/i18n.h>
 
 
-#ifndef WIN32
-#  include <unistd.h>
-#  include <signal.h>
-
-
 /*
  * Local functions.
  */
 
-void	sighandler(int);
-#endif /* !WIN32 */
 int	restart_job(const char *command, int job_id);
 int	set_job_attrs(const char *command, int job_id, int num_options,
 	              cups_option_t *options);
 
 
-/*
- * Globals...
- */
-
-char	tempfile[1024];		/* Temporary file for printing from stdin */
-
-
 /*
  * 'main()' - Parse options and send files for printing.
  */
@@ -73,21 +58,12 @@ main(int  argc,				/* I - Number of command-line arguments */
   int		num_copies;		/* Number of copies per file */
   int		num_files;		/* Number of files to print */
   const char	*files[1000];		/* Files to print */
-  int		num_dests;		/* Number of destinations */
-  cups_dest_t	*dests,			/* Destinations */
-		*dest;			/* Selected destination */
+  cups_dest_t	*dest;			/* Selected destination */
   int		num_options;		/* Number of options */
   cups_option_t	*options;		/* Options */
   int		end_options;		/* No more options? */
   int		silent;			/* Silent or verbose output? */
   char		buffer[8192];		/* Copy buffer */
-  ssize_t	bytes;			/* Bytes copied */
-  off_t		filesize;		/* Size of temp file */
-  int		temp;			/* Temporary file descriptor */
-#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;		/* Signal action */
-  struct sigaction oldaction;		/* Old signal action */
-#endif /* HAVE_SIGACTION && !HAVE_SIGSET*/
 
 
 #ifdef __sun
@@ -112,8 +88,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   silent      = 0;
   printer     = NULL;
-  num_dests   = 0;
-  dests       = NULL;
+  dest        = NULL;
   num_options = 0;
   options     = NULL;
   num_files   = 0;
@@ -179,10 +154,7 @@ main(int  argc,				/* I - Number of command-line arguments */
             if ((instance = strrchr(printer, '/')) != NULL)
 	      *instance++ = '\0';
 
-	    if (num_dests == 0)
-	      num_dests = cupsGetDests(&dests);
-
-            if ((dest = cupsGetDest(printer, instance, num_dests, dests)) != NULL)
+            if ((dest = cupsGetNamedDest(NULL, printer, instance)) != NULL)
 	    {
 	      for (j = 0; j < dest->num_options; j ++)
 	        if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
@@ -446,8 +418,12 @@ main(int  argc,				/* I - Number of command-line arguments */
               num_options = cupsAddOption("job-hold-until", "no-hold",
 	                                  num_options, &options);
 	    else if (!strcmp(val, "immediate"))
+	    {
+              num_options = cupsAddOption("job-hold-until", "no-hold",
+	                                  num_options, &options);
               num_options = cupsAddOption("job-priority", "100",
 	                                  num_options, &options);
+	    }
 	    else if (!strcmp(val, "restart"))
 	    {
 	      if (job_id < 1)
@@ -593,10 +569,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (printer == NULL)
   {
-    if (num_dests == 0)
-      num_dests = cupsGetDests(&dests);
-
-    if ((dest = cupsGetDest(NULL, NULL, num_dests, dests)) != NULL)
+    if ((dest = cupsGetNamedDest(NULL, NULL, NULL)) != NULL)
     {
       printer = dest->name;
 
@@ -625,7 +598,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     else
       val = "LPDEST";
 
-    if (printer && !cupsGetDest(printer, NULL, num_dests, dests))
+    if (printer && !cupsGetNamedDest(NULL, printer, NULL))
       _cupsLangPrintf(stderr,
 		      _("%s: Error - %s environment variable names "
 		        "non-existent destination \"%s\"!\n"),
@@ -644,73 +617,38 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (num_files > 0)
     job_id = cupsPrintFiles(printer, num_files, files, title, num_options, options);
-  else
+  else if ((job_id = cupsCreateJob(CUPS_HTTP_DEFAULT, printer,
+                                   title ? title : "(stdin)",
+                                   num_options, options)) > 0)
   {
-    num_files = 1;
+    http_status_t	status;		/* Write status */
+    const char		*format;	/* Document format */
+    ssize_t		bytes;		/* Bytes read */
 
-#ifndef WIN32
-#  if defined(HAVE_SIGSET)
-    sigset(SIGHUP, sighandler);
-    if (sigset(SIGINT, sighandler) == SIG_IGN)
-      sigset(SIGINT, SIG_IGN);
-    sigset(SIGTERM, sighandler);
-#  elif defined(HAVE_SIGACTION)
-    memset(&action, 0, sizeof(action));
-    action.sa_handler = sighandler;
 
-    sigaction(SIGHUP, &action, NULL);
-    sigaction(SIGINT, NULL, &oldaction);
-    if (oldaction.sa_handler != SIG_IGN)
-      sigaction(SIGINT, &action, NULL);
-    sigaction(SIGTERM, &action, NULL);
-#  else
-    signal(SIGHUP, sighandler);
-    if (signal(SIGINT, sighandler) == SIG_IGN)
-      signal(SIGINT, SIG_IGN);
-    signal(SIGTERM, sighandler);
-#  endif
-#endif /* !WIN32 */
+    if (cupsGetOption("raw", num_options, options))
+      format = CUPS_FORMAT_RAW;
+    else if ((format = cupsGetOption("document-format", num_options,
+                                     options)) == NULL)
+      format = CUPS_FORMAT_AUTO;
 
-    temp = cupsTempFd(tempfile, sizeof(tempfile));
+    status = cupsStartDocument(CUPS_HTTP_DEFAULT, printer, job_id, NULL,
+                               format, 1);
 
-    if (temp < 0)
+    while (status == HTTP_CONTINUE &&
+           (bytes = read(0, buffer, sizeof(buffer))) > 0)
+      status = cupsWriteRequestData(CUPS_HTTP_DEFAULT, buffer, bytes);
+
+    if (status != HTTP_CONTINUE)
     {
       _cupsLangPrintf(stderr,
-		      _("%s: Error - unable to create temporary file \"%s\" - %s\n"),
-        	      argv[0], tempfile, strerror(errno));
+		      _("%s: Error - unable to queue from stdin - %s\n"),
+		      argv[0], httpStatus(status));
       return (1);
     }
 
-    while ((bytes = read(0, buffer, sizeof(buffer))) > 0)
-      if (write(temp, buffer, bytes) < 0)
-      {
-	_cupsLangPrintf(stderr,
-		        _("%s: Error - unable to write to temporary file "
-			  "\"%s\" - %s\n"),
-        	        argv[0], tempfile, strerror(errno));
-        close(temp);
-        unlink(tempfile);
-	return (1);
-      }
-
-    filesize = lseek(temp, 0, SEEK_CUR);
-    close(temp);
-
-    if (filesize <= 0)
-    {
-      _cupsLangPrintf(stderr,
-		      _("%s: Error - stdin is empty, so no job has been sent.\n"),
-		      argv[0]);
-      unlink(tempfile);
-      return (1);
-    }
-
-    if (title)
-      job_id = cupsPrintFile(printer, tempfile, title, num_options, options);
-    else
-      job_id = cupsPrintFile(printer, tempfile, "(stdin)", num_options, options);
-
-    unlink(tempfile);
+    if (cupsFinishDocument(CUPS_HTTP_DEFAULT, printer) != IPP_OK)
+      job_id = 0;
   }
 
   if (job_id < 1)
@@ -734,13 +672,10 @@ int					/* O - Exit status */
 restart_job(const char *command,	/* I - Command name */
             int        job_id)		/* I - Job ID */
 {
-  http_t	*http;			/* HTTP connection to server */
   ipp_t		*request;		/* IPP request */
   char		uri[HTTP_MAX_URI];	/* URI for job */
 
 
-  http = httpConnectEncrypt(cupsServer(), ippPort(), cupsEncryption());
-
   request = ippNewRequest(IPP_RESTART_JOB);
 
   sprintf(uri, "ipp://localhost/jobs/%d", job_id);
@@ -751,7 +686,7 @@ restart_job(const char *command,	/* I - Command name */
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
                "requesting-user-name", NULL, cupsUser());
 
-  ippDelete(cupsDoRequest(http, request, "/jobs"));
+  ippDelete(cupsDoRequest(CUPS_HTTP_DEFAULT, request, "/jobs"));
 
   if (cupsLastError() > IPP_OK_CONFLICT)
   {
@@ -773,7 +708,6 @@ set_job_attrs(const char    *command,	/* I - Command name */
               int           num_options,/* I - Number of options */
 	      cups_option_t *options)	/* I - Options */
 {
-  http_t	*http;			/* HTTP connection to server */
   ipp_t		*request;		/* IPP request */
   char		uri[HTTP_MAX_URI];	/* URI for job */
 
@@ -781,8 +715,6 @@ set_job_attrs(const char    *command,	/* I - Command name */
   if (num_options == 0)
     return (0);
 
-  http = httpConnectEncrypt(cupsServer(), ippPort(), cupsEncryption());
-
   request = ippNewRequest(IPP_SET_JOB_ATTRIBUTES);
 
   sprintf(uri, "ipp://localhost/jobs/%d", job_id);
@@ -795,7 +727,7 @@ set_job_attrs(const char    *command,	/* I - Command name */
 
   cupsEncodeOptions(request, num_options, options);
 
-  ippDelete(cupsDoRequest(http, request, "/jobs"));
+  ippDelete(cupsDoRequest(CUPS_HTTP_DEFAULT, request, "/jobs"));
 
   if (cupsLastError() > IPP_OK_CONFLICT)
   {
@@ -807,29 +739,6 @@ set_job_attrs(const char    *command,	/* I - Command name */
 }
 
 
-#ifndef WIN32
 /*
- * 'sighandler()' - Signal catcher for when we print from stdin...
- */
-
-void
-sighandler(int s)			/* I - Signal number */
-{
- /*
-  * Remove the temporary file we're using to print from stdin...
-  */
-
-  unlink(tempfile);
-
- /*
-  * Exit...
-  */
-
-  exit(s);
-}
-#endif /* !WIN32 */
-
-
-/*
- * End of "$Id: lp.c 7721 2008-07-11 22:48:49Z mike $".
+ * End of "$Id: lp.c 8141 2008-11-18 00:42:34Z mike $".
  */
