@@ -1,9 +1,9 @@
 /*
- * "$Id: lpstat.c 8929 2009-12-15 22:40:37Z mike $"
+ * "$Id: lpstat.c 9259 2010-08-13 04:11:46Z mike $"
  *
- *   "lpstat" command for the Common UNIX Printing System (CUPS).
+ *   "lpstat" command for CUPS.
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -34,10 +34,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <cups/http-private.h>
-#include <cups/string.h>
-#include <cups/cups.h>
-#include <cups/i18n.h>
+#include <cups/globals.h>
 #include <cups/debug.h>
 
 
@@ -1423,7 +1420,7 @@ show_jobs(const char *dests,		/* I - Destinations */
 
       rank ++;
 
-      if (match_list(dests, dest) || match_list(users, username))
+      if (match_list(dests, dest) && match_list(users, username))
       {
         jobdate = localtime(&jobtime);
         snprintf(temp, sizeof(temp), "%s-%d", dest, jobid);
@@ -1518,7 +1515,7 @@ show_printers(const char  *printers,	/* I - Destinations */
   char		printer_uri[HTTP_MAX_URI],
 					/* Printer URI */
 		printer_state_time[255];/* Printer state time */
-  const char	*root;			/* Server root directory... */
+  _cups_globals_t *cg = _cupsGlobals();	/* Global data */
   static const char *pattrs[] =		/* Attributes we need for printers... */
 		{
 		  "printer-name",
@@ -1536,16 +1533,14 @@ show_printers(const char  *printers,	/* I - Destinations */
 		};
   static const char *jattrs[] =		/* Attributes we need for jobs... */
 		{
-		  "job-id"
+		  "job-id",
+		  "job-state"
 		};
 
 
   DEBUG_printf(("show_printers(printers=\"%s\", num_dests=%d, dests=%p, "
                 "long_status=%d)\n", printers, num_dests, dests, long_status));
 
-  if ((root = getenv("CUPS_SERVERROOT")) == NULL)
-    root = CUPS_SERVERROOT;
-
   if (printers != NULL && !strcmp(printers, "all"))
     printers = NULL;
 
@@ -1820,10 +1815,11 @@ show_printers(const char  *printers,	/* I - Destinations */
 	    if (make_model && strstr(make_model, "System V Printer"))
 	      _cupsLangPrintf(stdout,
 	                      _("\tInterface: %s/interfaces/%s\n"),
-			      root, printer);
+			      cg->cups_serverroot, printer);
 	    else if (make_model && !strstr(make_model, "Raw Printer"))
 	      _cupsLangPrintf(stdout,
-	                      _("\tInterface: %s/ppd/%s.ppd\n"), root, printer);
+	                      _("\tInterface: %s/ppd/%s.ppd\n"),
+			      cg->cups_serverroot, printer);
           }
 	  _cupsLangPuts(stdout, _("\tOn fault: no alert\n"));
 	  _cupsLangPuts(stdout, _("\tAfter fault: continue\n"));
@@ -1933,10 +1929,11 @@ show_printers(const char  *printers,	/* I - Destinations */
 		if (make_model && strstr(make_model, "System V Printer"))
 		  _cupsLangPrintf(stdout,
 	                	  _("\tInterface: %s/interfaces/%s\n"),
-				  root, printer);
+				  cg->cups_serverroot, printer);
 		else if (make_model && !strstr(make_model, "Raw Printer"))
 		  _cupsLangPrintf(stdout,
-	                	  _("\tInterface: %s/ppd/%s.ppd\n"), root, printer);
+	                	  _("\tInterface: %s/ppd/%s.ppd\n"),
+				  cg->cups_serverroot, printer);
               }
 	      _cupsLangPuts(stdout, _("\tOn fault: no alert\n"));
 	      _cupsLangPuts(stdout, _("\tAfter fault: continue\n"));
@@ -2010,5 +2007,5 @@ show_scheduler(void)
 
 
 /*
- * End of "$Id: lpstat.c 8929 2009-12-15 22:40:37Z mike $".
+ * End of "$Id: lpstat.c 9259 2010-08-13 04:11:46Z mike $".
  */
